@@ -2,27 +2,26 @@ use std::any::Any;
 use std::collections::VecDeque;
 use std::convert::Infallible;
 use std::marker::PhantomData;
+use std::os::unix::io::{AsFd, BorrowedFd, OwnedFd};
 use std::sync::{atomic::Ordering, Arc, Condvar, Mutex};
 use std::task;
 
-use nix::Error;
 use wayland_backend::{
     client::{Backend, ObjectData, ObjectId, ReadEventsGuard, WaylandError},
-    io_lifetimes::OwnedFd,
     protocol::{Argument, Message},
 };
 
 use crate::{conn::SyncData, Connection, DispatchError, Proxy};
 
-/// A trait providing an implementation for handling events a proxy through an [`EventQueue`].
+/// A trait for handlers of proxies' events delivered to an [`EventQueue`].
 ///
 /// ## General usage
 ///
 /// You need to implement this trait on your `State` for every type of Wayland object that will be processed
 /// by the [`EventQueue`] working with your `State`.
 ///
-/// You can have different implementations of the trait for the same interface but different `UserData` type,
-/// this way the events for a given object will be processed by the adequate implementation depending on
+/// You can have different implementations of the trait for the same interface but different `UserData` type.
+/// This way the events for a given object will be processed by the adequate implementation depending on
 /// which `UserData` was assigned to it at creation.
 ///
 /// The way this trait works is that the [`Dispatch::event()`] method will be invoked by the event queue for
@@ -31,7 +30,7 @@ use crate::{conn::SyncData, Connection, DispatchError, Proxy};
 ///
 /// In the rare case of an interface with *events* creating new objects (in the core protocol, the only
 /// instance of this is the `wl_data_device.data_offer` event), you'll need to implement the
-/// [`Dispatch::event_created_child()`] method. See the [`event_created_child!`](event_created_child!) macro
+/// [`Dispatch::event_created_child()`] method. See the [`event_created_child!`](macro.event_created_child.html) macro
 /// for a simple way to do this.
 ///
 /// ## Modularity
@@ -40,7 +39,7 @@ use crate::{conn::SyncData, Connection, DispatchError, Proxy};
 /// that is generic over the last type argument, as illustrated below. Users will then be able to
 /// automatically delegate their implementation to yours using the [`delegate_dispatch!`] macro.
 ///
-/// As a result, when your implementation is instanciated, the last type parameter `State` will be the state
+/// As a result, when your implementation is instantiated, the last type parameter `State` will be the state
 /// struct of the app using your generic implementation. You can put additional trait constraints on it to
 /// specify an interface between your module and downstream code, as illustrated in this example:
 ///
@@ -52,8 +51,8 @@ use crate::{conn::SyncData, Connection, DispatchError, Proxy};
 /// struct DelegateToMe;
 ///
 /// /// The user data relevant for your implementation.
-/// /// When providing delegate implementation, it is recommended to use your own type here, even if it is
-/// /// just a unit struct: using () would cause a risk of clashing with an other such implementation.
+/// /// When providing a delegate implementation, it is recommended to use your own type here, even if it is
+/// /// just a unit struct: using () would cause a risk of clashing with another such implementation.
 /// struct MyUserData;
 ///
 /// // Now a generic implementation of Dispatch, we are generic over the last type argument instead of using
@@ -89,6 +88,8 @@ use crate::{conn::SyncData, Connection, DispatchError, Proxy};
 /// implementation of [`Dispatch`] cannot be used directly as the dispatching state, as rustc
 /// currently fails to understand that it also provides `Dispatch<I, U, Self>` (assuming all other
 /// trait bounds are respected as well).
+///
+/// [`delegate_dispatch!`]: crate::delegate_dispatch
 pub trait Dispatch<I, UserData, State = Self>
 where
     Self: Sized,
@@ -118,8 +119,8 @@ where
     /// Method used to initialize the user-data of objects created by events
     ///
     /// If the interface does not have any such event, you can ignore it. If not, the
-    /// [`event_created_child!`](event_created_child!) macro is provided for overriding it.
-    #[cfg_attr(coverage, no_coverage)]
+    /// [`event_created_child!`](macro.event_created_child.html) macro is provided for overriding it.
+    #[cfg_attr(coverage, coverage(off))]
     fn event_created_child(opcode: u16, _qhandle: &QueueHandle<State>) -> Arc<dyn ObjectData> {
         panic!(
             "Missing event_created_child specialization for event opcode {} of {}",
@@ -193,7 +194,7 @@ type QueueCallback<State> = fn(
 struct QueueEvent<State>(QueueCallback<State>, Message<ObjectId, OwnedFd>, Arc<dyn ObjectData>);
 
 impl<State> std::fmt::Debug for QueueEvent<State> {
-    #[cfg_attr(coverage, no_coverage)]
+    #[cfg_attr(coverage, coverage(off))]
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         f.debug_struct("QueueEvent").field("msg", &self.1).finish_non_exhaustive()
     }
@@ -343,12 +344,19 @@ impl<State> EventQueueInner<State> {
 }
 
 impl<State> std::fmt::Debug for EventQueue<State> {
-    #[cfg_attr(coverage, no_coverage)]
+    #[cfg_attr(coverage, coverage(off))]
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         f.debug_struct("EventQueue").field("handle", &self.handle).finish_non_exhaustive()
     }
 }
 
+impl<State> AsFd for EventQueue<State> {
+    /// Provides fd from [`Backend::poll_fd`] for polling.
+    fn as_fd(&self) -> BorrowedFd<'_> {
+        self.conn.as_fd()
+    }
+}
+
 impl<State> EventQueue<State> {
     pub(crate) fn new(conn: Connection) -> Self {
         let inner = Arc::new(Mutex::new(EventQueueInner {
@@ -391,14 +399,8 @@ impl<State> EventQueue<State> {
 
         self.conn.flush()?;
 
-        let guard = self.conn.prepare_read()?;
-
-        // we need to check the queue again, just in case another thread did a read between
-        // dispatch_pending and prepare_read
-        if self.handle.inner.lock().unwrap().queue.is_empty() {
+        if let Some(guard) = self.conn.prepare_read() {
             crate::conn::blocking_read(guard)?;
-        } else {
-            drop(guard);
         }
 
         self.dispatch_pending(data)
@@ -421,7 +423,7 @@ impl<State> EventQueue<State> {
                 crate::protocol::wl_display::Request::Sync {},
                 Some(done.clone()),
             )
-            .map_err(|_| WaylandError::Io(Error::EPIPE.into()))?;
+            .map_err(|_| WaylandError::Io(rustix::io::Errno::PIPE.into()))?;
 
         let mut dispatched = 0;
 
@@ -439,11 +441,15 @@ impl<State> EventQueue<State> {
     /// you'll then need to dispatch them from the event queue using
     /// [`EventQueue::dispatch_pending()`](EventQueue::dispatch_pending).
     ///
+    /// If this method returns `None`, you should invoke ['dispatch_pending()`](EventQueue::dispatch_pending)
+    /// before trying to invoke it again.
+    ///
     /// If you don't need to manage multiple event sources, see
     /// [`blocking_dispatch()`](EventQueue::blocking_dispatch) for a simpler mechanism.
     ///
     /// This method is identical to [`Connection::prepare_read()`].
-    pub fn prepare_read(&self) -> Result<ReadEventsGuard, WaylandError> {
+    #[must_use]
+    pub fn prepare_read(&self) -> Option<ReadEventsGuard> {
         self.conn.prepare_read()
     }
 
@@ -466,9 +472,8 @@ impl<State> EventQueue<State> {
         //
         // We purposefully ignore the possible error, as that would make us early return in a way that might
         // lose events, and the potential socket error will be caught in other places anyway.
-        let _ = backend.backend.dispatch_inner_queue();
+        let mut dispatched = backend.backend.dispatch_inner_queue().unwrap_or_default();
 
-        let mut dispatched = 0;
         while let Some(QueueEvent(cb, msg, odata)) = Self::try_next(&qhandle.inner) {
             cb(backend, msg, data, odata, qhandle)?;
             dispatched += 1;
@@ -581,7 +586,7 @@ pub struct QueueFreezeGuard<'a, State> {
 }
 
 impl<State> std::fmt::Debug for QueueHandle<State> {
-    #[cfg_attr(coverage, no_coverage)]
+    #[cfg_attr(coverage, coverage(off))]
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         f.debug_struct("QueueHandle").field("inner", &Arc::as_ptr(&self.inner)).finish()
     }
@@ -688,7 +693,7 @@ where
 }
 
 impl<I: Proxy, U: std::fmt::Debug, State> std::fmt::Debug for QueueProxyData<I, U, State> {
-    #[cfg_attr(coverage, no_coverage)]
+    #[cfg_attr(coverage, coverage(off))]
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         f.debug_struct("QueueProxyData").field("udata", &self.udata).finish()
     }
